/*******************************************************************************
*
* E M B E D D E D   W I Z A R D   P R O J E C T
*
*                                                Copyright (c) TARA Systems GmbH
*                                    written by Paul Banach and Manfred Schweyer
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* TARA Systems hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   This file is part of the interface (glue layer) between an Embedded Wizard
*   generated UI application and the board support package (BSP) of a dedicated
*   target.
*   This template is responsible to initialize the system clock of the hardware,
*   to provide a timer tick for the Embedded Wizard UI application and to get
*   access to the realtime clock (RTC). Additionally, this file contains some
*   performance measurements to analyse the CPU usage of the UI application.
*
*******************************************************************************/

#include <stdint.h>
#include <stdbool.h>
#include "iodefine.h"
#include "rtc.h"

#include "ewconfig.h"
#include "ew_bsp_clock.h"


#define RtcTicksPerSecond    ( 1UL )
#define RtcTicksPerMinute    ( RtcTicksPerSecond * 60UL )
#define RtcTicksPerHour      ( RtcTicksPerMinute * 60UL )
#define RtcTicksPerDay       ( RtcTicksPerHour   * 24UL )

const unsigned long DaysToMonth[] =
{
  0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365
};

const unsigned long DaysToMonthInLeapYear[] =
{
  0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366
};

/* helper function to initialize measuring unit (cycle counter) */
static void CycleCounterInit( void )
{
  /* set the protect register to write enable and key */
  SYSTEM.PRCR.WORD = 0xa502U;

  /* Bring module out of stop state */
  MSTP(CMT1) = 0U;

  /* stop counter */
  CMT.CMSTR0.BIT.STR1 = 0U;

  /* set the match value */
  CMT1.CMCOR = 10U;

  /* set the clock divider to 1 */
  CMT1.CMCR.BIT.CKS = 0U;

  /* enable interrupt on match */
  CMT1.CMCR.BIT.CMIE = 1U;

  /* clear counter. */
  CMT1.CMCNT = 0U;

  /* clear any previously pending interrupts */
  IR(CMT1, CMI1) = 0U;

  /* set interrupt priority */
  IPR(CMT1, CMI1) = 1U;

  /* enable compare match interrupt */
  IEN(CMT1, CMI1) = 1U;

  /* start counter. */
  CMT.CMSTR0.BIT.STR1 = 1U;

  /* set the protect register to write disable and key */
  SYSTEM.PRCR.WORD = 0xa500U;

}


/*******************************************************************************
* FUNCTION:
*   EwBspClockInit
*
* DESCRIPTION:
*   Initialises the system clock and the real time clock.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspClockInit( void )
{
  /* set the protect register to write enable and key */
  SYSTEM.PRCR.WORD = 0xa502U;

  /* Bring module out of stop state */
  MSTP(CMT0) = 0U;

  /* stop counter */
  CMT.CMSTR0.BIT.STR0 = 0U;

  /* set the match value */
  CMT0.CMCOR = 5859U;

  /* set the clock divider to 1 */
  CMT0.CMCR.BIT.CKS = 0U;

  /* enable interrupt on match */
  CMT0.CMCR.BIT.CMIE = 1U;

  /* clear counter. */
  CMT0.CMCNT = 0U;

  /* clear any previously pending interrupts */
  IR(CMT0, CMI0) = 0U;

  /* set interrupt priority */
  IPR(CMT0, CMI0) = 5U;

  /* enable compare match interrupt */
  IEN(CMT0, CMI0) = 1U;

  /* start counter. */
  CMT.CMSTR0.BIT.STR0 = 1U;

  /* set the protect register to write disable and key */
  SYSTEM.PRCR.WORD = 0xa500U;

  /* initialize system cycle counter (also used for display synchronisation) */
  CycleCounterInit();

  /* init RTC */
  rtc_init();
}


/*******************************************************************************
* FUNCTION:
*   EwBspClockGetTime
*
* DESCRIPTION:
*   Returns the current time in seconds since 01.01.1970.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   The current time in seconds since 01.01.1970.
*
*******************************************************************************/
unsigned long EwBspClockGetTime( void )
{
  tm_t RtcTime;
  rtc_get_time(&RtcTime);

  unsigned long   days;
  unsigned long   year   = RtcTime.tm_year - 100;
  unsigned long   month  = RtcTime.tm_mon + 1;
  unsigned long   day    = RtcTime.tm_mday;
  unsigned long   hour   = RtcTime.tm_hour;
  unsigned long   minute = RtcTime.tm_min;
  unsigned long   second = RtcTime.tm_sec;
  unsigned long   timeInSeconds;

  /* year is the number of years since 2000 */
  /* calculate number of days since 01.01.1970 until begin of current
     year including the additional days of leap years */

  days = (year + 30) * 365 + (year + 27) / 4;

  if (year & 0x3)
  {
    days += DaysToMonth[month - 1];
  }
  else
  {
    days += DaysToMonthInLeapYear[month - 1];
  }

  days += day;

  timeInSeconds =
    days   * RtcTicksPerDay +
    hour   * RtcTicksPerHour +
    minute * RtcTicksPerMinute +
    second;

  return timeInSeconds;
}


/*******************************************************************************
* FUNCTION:
*   EwBspClockSetTime
*
* DESCRIPTION:
*   Sets the given time in seconds since 01.01.1970 at real time clock (RTC).
*
* ARGUMENTS:
*   aTime - the time in seconds since 01.01.1970 to set in real time clock.
*
* RETURN VALUE:
*   None.
*
*******************************************************************************/
void EwBspClockSetTime( unsigned long aTime )
{
  #define LeapYearsFromBeginToNext400Year   7
  #define DaysPer4Years       ( 4 * 365 + 1 )

  tm_t RtcTime;

  unsigned int year;
  unsigned int year4;
  unsigned int month;
  unsigned int day;
  unsigned int hour;
  unsigned int minute;
  unsigned int second;

  hour    = ( unsigned int ) (( aTime / RtcTicksPerHour    ) % 24 );
  minute  = ( unsigned int ) (( aTime / RtcTicksPerMinute  ) % 60 );
  second  = ( unsigned int ) (( aTime / RtcTicksPerSecond  ) % 60 );

  /* calculation of the date */
  day       = ( unsigned int ) ( aTime / RtcTicksPerDay + 365);

  if ( day >= 48213 )
    day++;

  year4     = day / DaysPer4Years;
  day      -= year4 * DaysPer4Years;
  year      = day / 365;

  if ( year == 4 )
  {
    year = 3;

    if ( !( year4 % 100 == LeapYearsFromBeginToNext400Year )
      && ( year4 %  25 == LeapYearsFromBeginToNext400Year ))
      year = 4;
  }

  day  -= year * 365;
  year  = year4 * 4 + year + 1969;

  /* all months have less than 32 days, days >> 5 is a good estimate
    for the month. */

  month = day >> 5;

  /* correct month if necessary */
  if ( ( year % 4 != 0 ) ||
    (( year % 100 == 0 ) && ( year % 400 != 0 )) )
  {
    /* no leap year */
    if ( day >= DaysToMonth[ month + 1 ] )
      month++;

    day -= DaysToMonth[ month ];
  }
  else
  {
    /* leap year */
    if ( day >= DaysToMonthInLeapYear[ month + 1 ] )
      month++;

    day -= DaysToMonthInLeapYear[ month ];
  }

  /* Setup RTC structure */
  RtcTime.tm_sec = second;
  RtcTime.tm_min = minute;
  RtcTime.tm_hour = hour;
  RtcTime.tm_mday = day + 1;
  RtcTime.tm_mon = month;
  RtcTime.tm_year = year - 2000 + 100;
  RtcTime.tm_wday = 0;
  RtcTime.tm_yday = 0;
  RtcTime.tm_isdst = 0;

  /* Initialize RTC */
  rtc_set_time(&RtcTime);
}


/*******************************************************************************
* FUNCTION:
*   EwBspClockGetCpuLoad
*
* DESCRIPTION:
*   Returns the current CPU load as percent value.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   The current CPU load.
*
*******************************************************************************/
 int EwBspClockGetCpuLoad( void )
{
  int cpuLoad = 0;

  return cpuLoad;
}


/*******************************************************************************
* FUNCTION:
*   EwBspClockCpuLoadSetActive
*
* DESCRIPTION:
*   Starts the CPU load counting. Call this function whenever CPU processing
*   is needed.
*
* ARGUMENTS:
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspClockCpuLoadSetActive( void )
{
	/* Not implemented */
}


/*******************************************************************************
* FUNCTION:
*   EwBspClockCpuLoadSetIdle
*
* DESCRIPTION:
*   Stops the CPU load counting. Call this function whenever CPU processing is
*   currently not needed since the program execution is waiting for some system
*   event.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspClockCpuLoadSetIdle( void )
{
	/* Not implemented */
}


/* mli */
