/*******************************************************************************
*
* E M B E D D E D   W I Z A R D   P R O J E C T
*
*                                                Copyright (c) TARA Systems GmbH
*                                    written by Paul Banach and Manfred Schweyer
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* TARA Systems hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   This file is part of the interface (glue layer) between an Embedded Wizard
*   generated UI application and the board support package (BSP) of a dedicated
*   target.
*   This template is responsible to initialize the display hardware of the board
*   and to provide the necessary access to update the display content.
*   The color format of the framebuffer has to correspond to the color format
*   of the Graphics Engine.
*
*   Important: This file is intended to be used as a template. Please adapt the
*   implementation according your particular hardware.
*
*******************************************************************************/

#include <stdlib.h>
#include <stdint.h>
#include <stdbool.h>
#include <string.h>
#include "iodefine.h"

#include "ewconfig.h"
#include "ewrte.h"
#include "ewgfx.h"
#include "ewextgfx.h"
#include "ewgfxdefs.h"

#include "ew_bsp_display.h"
#include "ew_bsp_clock.h"

/* LCD settings */
#define LCD_VERT_SYNC_AREA              2
#define LCD_VERT_BACK_PORCH             7
#define LCD_VERT_ACTIVE_AREA            272
#define LCD_VERT_FRONT_PORCH            8
#define LCD_VERT_TOTAL                  289


#if EW_USE_DOUBLE_BUFFER == 1

  static volatile uint32_t      PendingFramebuffer = 0;

#else

  static volatile unsigned long VSyncTime          = 0;
  static volatile unsigned long VSyncDuration      = 0;

#endif

volatile unsigned long        CycleCounter       = 0;


void GLCDC_VPOS_IRQHandler(void)
{
  /* reset the VPOS status register */
  GLCDC.STCLR.BIT.VPOSCLR = 1;

  #if EW_USE_DOUBLE_BUFFER == 1

    /* as soon as V-sync interrupt occurs, no buffer is pending */
    if ( PendingFramebuffer )
  	  PendingFramebuffer = 0;

  #else

    unsigned long curTicks = CycleCounter;
    VSyncDuration = curTicks - VSyncTime;
    VSyncTime = curTicks;

  #endif
}


/* Helper function to sync on a certain line number updated by the display.
   This function is necessary when the system is used in single framebuffer mode. */
#if EW_USE_DOUBLE_BUFFER == 0
static void SyncOnLine( int aLine )
{
  unsigned long ticksTillRequestedLine;
  unsigned long localVSyncTime;

  /* line interrupt is configured to begin of front porch area - we have to add vertical
     front porch and vertical back porch to start on the desired visible line */
  aLine += ( LCD_VERT_TOTAL - LCD_VERT_ACTIVE_AREA );

  /* calculate time tick that corresponds to requested line */
  ticksTillRequestedLine = aLine * ( VSyncDuration / LCD_VERT_TOTAL );

  CPU_LOAD_SET_IDLE();

  /* if requested line is missed, wait for next display frame (next V-sync) */
  while (( CycleCounter - VSyncTime ) > ticksTillRequestedLine )
    ;

  /* wait for requested line in current display frame */
  localVSyncTime = VSyncTime;
  while (( CycleCounter - localVSyncTime ) < ticksTillRequestedLine )
    ;

  CPU_LOAD_SET_ACTIVE();
}
#endif


/*******************************************************************************
* FUNCTION:
*   EwBspDisplayInit
*
* DESCRIPTION:
*   The function EwBspDisplayInit initializes the display hardware and returns
*   the display parameter.
*
* ARGUMENTS:
*   aGuiWidth,
*   aGuiHeight   - Size of the GUI in pixel.
*   aDisplayInfo - Display info data structure.
*
* RETURN VALUE:
*   Returns 1 if successful, 0 otherwise.
*
*******************************************************************************/
int EwBspDisplayInit( int aGuiWidth, int aGuiHeight, XDisplayInfo* aDisplayInfo )
{
  EW_UNUSED_ARG( aGuiWidth );
  EW_UNUSED_ARG( aGuiHeight );

  /* set bus master priority of GLCDC higher than DRW2D to avoid buffer underflows */
  unsigned long* EMAPCR = (unsigned long*)0x000C5800;
  *EMAPCR = 0x00042130;

  /* allow writing to PFSWE bit */
  MPC.PWPR.BIT.B0WI = 0U;

  /* enable writing to PFS registers. */
  MPC.PWPR.BIT.PFSWE = 1U;

  /* set LCD_CLK pin to peripheral i/o */
  MPC.PB5PFS.BYTE = 0x25U;
  PORTB.PMR.BIT.B5 = 1U;

  /* set LCD_TCON0 pin to peripheral i/o */
  MPC.PB4PFS.BYTE = 0x25U;
  PORTB.PMR.BIT.B4 = 1U;

  /* set LCD_TCON2 pin to peripheral i/o */
  MPC.PB2PFS.BYTE = 0x25U;
  PORTB.PMR.BIT.B2 = 1U;

  /* set LCD_TCON3 pin to peripheral i/o */
  MPC.PB1PFS.BYTE = 0x25U;
  PORTB.PMR.BIT.B1 = 1U;

  /* set LCD_DATA0 pin to peripheral i/o */
  MPC.PB0PFS.BYTE = 0x25U;
  PORTB.PMR.BIT.B0 = 1U;

  /* set LCD_DATA1 pin to peripheral i/o */
  MPC.PA7PFS.BYTE = 0x25U;
  PORTA.PMR.BIT.B7 = 1U;

  /* set LCD_DATA2 pin to peripheral i/o */
  MPC.PA6PFS.BYTE = 0x25U;
  PORTA.PMR.BIT.B6 = 1U;

  /* set LCD_DATA3 pin to peripheral i/o */
  MPC.PA5PFS.BYTE = 0x25U;
  PORTA.PMR.BIT.B5 = 1U;

  /* set LCD_DATA4 pin to peripheral i/o */
  MPC.PA4PFS.BYTE = 0x25U;
  PORTA.PMR.BIT.B4 = 1U;

  /* set LCD_DATA5 pin to peripheral i/o */
  MPC.PA3PFS.BYTE = 0x25U;
  PORTA.PMR.BIT.B3 = 1U;

  /* set LCD_DATA6 pin to peripheral i/o */
  MPC.PA2PFS.BYTE = 0x25U;
  PORTA.PMR.BIT.B2 = 1U;

  /* set LCD_DATA7 pin to peripheral i/o */
  MPC.PA1PFS.BYTE = 0x25U;
  PORTA.PMR.BIT.B1 = 1U;

  /* set LCD_DATA8 pin to peripheral i/o */
  MPC.PA0PFS.BYTE = 0x25U;
  PORTA.PMR.BIT.B0 = 1U;

  /* set LCD_DATA9 pin to peripheral i/o */
  MPC.PE7PFS.BYTE = 0x25U;
  PORTE.PMR.BIT.B7 = 1U;

  /* set LCD_DATA10 pin to peripheral i/o */
  MPC.PE6PFS.BYTE = 0x25U;
  PORTE.PMR.BIT.B6 = 1U;

  /* set LCD_DATA11 pin to peripheral i/o */
  MPC.PE5PFS.BYTE = 0x25U;
  PORTE.PMR.BIT.B5 = 1U;

  /* set LCD_DATA12 pin to peripheral i/o */
  MPC.PE4PFS.BYTE = 0x25U;
  PORTE.PMR.BIT.B4 = 1U;

  /* set LCD_DATA13 pin to peripheral i/o */
  MPC.PE3PFS.BYTE = 0x25U;
  PORTE.PMR.BIT.B3 = 1U;

  /* set LCD_DATA14 pin to peripheral i/o */
  MPC.PE2PFS.BYTE = 0x25U;
  PORTE.PMR.BIT.B2 = 1U;

  /* set LCD_DATA15 pin to peripheral i/o */
  MPC.PE1PFS.BYTE = 0x25U;
  PORTE.PMR.BIT.B1 = 1U;

  /* disable writing to PFS registers. */
  MPC.PWPR.BIT.PFSWE = 0U;

  /* disable writing of PFSWE bit. */
  MPC.PWPR.BIT.B0WI = 1U;

  /* set up reset line and bring display out of reset */
  PORT6.PMR.BIT.B3 = 0U;  /* mode to gpio */
  PORT6.PDR.BIT.B3 = 1U;   /* port direction output */
  PORT6.ODR0.BIT.B6 = 0U;  /* cmos */
  PORT6.PODR.BIT.B3 = 1U; /* bring display out of reset */

  /* set backlight pin as gpio output and switch backlight on */
  PORT6.PMR.BIT.B6 = 0U;  /* mode to gpio */
  PORT6.PDR.BIT.B6 = 1U;  /* port direction output */
  PORT6.ODR1.BIT.B4 = 0U;  /* cmos */
  PORT6.PODR.BIT.B6 = 1U;  /* backlight on */

  /* enable writing to clock control bit */
  SYSTEM.PRCR.WORD = 0xa502U;

  /* supply the peripheral clock to the glcd module */
  MSTP(GLCDC) = 0U;

  /* disable writing to clock control bit */
  SYSTEM.PRCR.WORD = 0xa500;

  /* release glcd module from a software reset status */
  GLCDC.BGEN.BIT.SWRST = 1U;

  /* set the dot clock to pll clock */
  GLCDC.PANELCLK.BIT.CLKSEL = 1U;

  /* set clock division ratio */
  GLCDC.PANELCLK.BIT.DCDR = 0x18U;

  /* selects pixel clock output to no division */
  GLCDC.PANELCLK.BIT.PIXSEL = 0U;

  /* enable panel clock LCD_CLK output */
  GLCDC.PANELCLK.BIT.CLKEN = 1U;

  /* waiting for supply of panel clock(LCD_CLK) and pixel clock(PXCLK)
   * the BGMON.SWRST bit is sampled with PXCLK so if the CLKEN bit is set,
   * the BGEN.SWRST bit is reflected on the BGMON.SWRST bit. */
  while (0U == GLCDC.BGMON.BIT.SWRST)
  {
  __asm("NOP");
  }

  /* set the panel signal timing */

  /* the outputs of LCD_DATA are synchronized with rising edges of LCD_CLK */
  GLCDC.CLKPHASE.BIT.LCDEDG = 0U;

  /* the output of LCD_TCON0 is synchronized with rising edges of LCD_CLK */
  GLCDC.CLKPHASE.BIT.TCON0EDG = 0U;

  /* the output of LCD_TCON2 is synchronized with rising edges of LCD_CLK */
  GLCDC.CLKPHASE.BIT.TCON2EDG = 0U;

  /* the output of LCD_TCON3 is synchronized with rising edges of LCD_CLK */
  GLCDC.CLKPHASE.BIT.TCON3EDG = 0U;

  /* horiz synch timing offset is 1 pixel */
  GLCDC.TCONTIM.BIT.OFFSET = 0U;

  /* vert synch timing offset is 1 pixel */
  GLCDC.TCONTIM.BIT.HALF = 0U;

  /* output signal select STHA */
  GLCDC.TCONSTHA2.BIT.SEL = 2U;

  /* invert the STHy signal */
  GLCDC.TCONSTHA2.BIT.INV = 1U;

  /* set hsync beginning position to no delay */
  GLCDC.TCONSTHA1.BIT.HS = 0U;

  /* select input hsync as reference */
  GLCDC.TCONSTHA2.BIT.HSSEL = 0U;

  /* STHy signal pulse width 1 pixel */
  GLCDC.TCONSTHA1.BIT.HW = 1U;

  /* vsync to STVA */
  GLCDC.TCONSTVA2.BIT.SEL = 0U;

  /* invert vsync */
  GLCDC.TCONSTVA2.BIT.INV = 1U;

  /* no vsync delay */
  GLCDC.TCONSTVA1.BIT.VS = 0U;

  /* pulse width 1 */
  GLCDC.TCONSTVA1.BIT.VW = 1U;

  /* output signal data enable */
  GLCDC.TCONSTHB2.BIT.SEL = 7U;

  /* don't invert data enable */
  GLCDC.TCONDE.BIT.INV = 0U;

  /* hsync delay */
  GLCDC.TCONSTHB1.BIT.HS = 0x29U;

  /* horiz width */
  GLCDC.TCONSTHB1.BIT.HW = 480;

  /* hsync is reference */
  GLCDC.TCONSTHB2.BIT.HSSEL = 0U;

  /* vsync 9 lines delay */
  GLCDC.TCONSTVB1.BIT.VS = 9U;

  /* vert height aHeight */
  GLCDC.TCONSTVB1.BIT.VW = 272;

  /* hsync period */
  GLCDC.BGPERI.BIT.FH = 0x20dU;

  /* vsync period */
  GLCDC.BGPERI.BIT.FV = 0x120U;

  /* hsync assert position */
  GLCDC.BGSYNC.BIT.HP = 3U;

  /* vsync assert position */
  GLCDC.BGSYNC.BIT.VP = 7U;

  /* horiz active start pixel */
  GLCDC.BGHSIZE.BIT.HP = 0x2cU;

  /* vert active start position */
  GLCDC.BGVSIZE.BIT.VP = 0x10U;

  /* horiz active pixel width */
  GLCDC.BGHSIZE.BIT.HW = 480;

  /* vert active pixel height */
  GLCDC.BGVSIZE.BIT.VW = 272;

  /* set background colour to black */
  GLCDC.BGCOLOR.BIT.R = 0U;
  GLCDC.BGCOLOR.BIT.G = 0U;
  GLCDC.BGCOLOR.BIT.B = 0U;

  /* height of graphical area */
  GLCDC.GR2AB2.BIT.GRCVW = 272;

  /* width of graphical area */
  GLCDC.GR2AB3.BIT.GRCHW = 480;

  #if ( EW_FRAME_BUFFER_COLOR_FORMAT == EW_FRAME_BUFFER_COLOR_FORMAT_RGB565 )

    /* frame buffer 2 565 format */
    GLCDC.GR2FLM6.BIT.FORMAT = 0U;

    /* line offset bytes gr2 */
    GLCDC.GR2FLM3.BIT.LNOFF = 960;

    /* set number of 64 byte transfers, gr2 */
    GLCDC.GR2FLM5.BIT.DATANUM = 14;

    /* set number of lines in display, gr2 */
    GLCDC.GR2FLM5.BIT.LNNUM = 271;

  #elif ( EW_FRAME_BUFFER_COLOR_FORMAT == EW_FRAME_BUFFER_COLOR_FORMAT_Index8 )

    /* frame buffer 2 Clut 8 */
    GLCDC.GR2FLM6.BIT.FORMAT = 5U;

    /* line offset bytes gr2 */
    GLCDC.GR2FLM3.BIT.LNOFF = 512;

    /* set number of 64 byte transfers, gr2 */
    GLCDC.GR2FLM5.BIT.DATANUM = 7;

    /* set number of lines in display, gr2 */
    GLCDC.GR2FLM5.BIT.LNNUM = 271;

  #elif ( EW_FRAME_BUFFER_COLOR_FORMAT == EW_FRAME_BUFFER_COLOR_FORMAT_LumA44 )

    /* frame buffer 2 Clut 8 */
    GLCDC.GR2FLM6.BIT.FORMAT = 5U;

    /* line offset bytes gr2 */
    GLCDC.GR2FLM3.BIT.LNOFF = 512;

    /* set number of 64 byte transfers, gr2 */
    GLCDC.GR2FLM5.BIT.DATANUM = 7;

    /* set number of lines in display, gr2 */
    GLCDC.GR2FLM5.BIT.LNNUM = 271;

  #endif

  /* pixel format 565 */
  GLCDC.OUTSET.BIT.FORMAT = 2U;
  GLCDC.PANELDTHA.BIT.FORM = 2U;

  /* set start address of gr2 buffer */
  GLCDC.GR2FLM2 = (uint32_t)EW_FRAME_BUFFER_ADDR;

  /* set background colour */
  GLCDC.GR2BASE.BIT.R = 0U;
  GLCDC.GR2BASE.BIT.G = 0U;
  GLCDC.GR2BASE.BIT.B = 0U;

  /* graphical area vertical start position */
  GLCDC.GR2AB2.BIT.GRCVS = 9U;

  /* graphical area horizontal start position */
  GLCDC.GR2AB3.BIT.GRCHS = 0x29U;

  /* frame not displayed */
  GLCDC.GR2AB1.BIT.GRCDISPON = 0U;

  /* set show current graphics */
  GLCDC.GR2AB1.BIT.DISPSEL = 2U;

  /* disable chroma control */
  GLCDC.GR2AB7.BIT.CKON = 0U;

  /* configure the output control block */
  GLCDC.OUTSET.BIT.ENDIANON = 0U;

  /* byte order RGB */
  GLCDC.OUTSET.BIT.SWAPON = 0U;

  /* no pixel clock division */
  GLCDC.OUTSET.BIT.FRQSEL = 0U;

  /* rounding mode is truncate */
  GLCDC.PANELDTHA.BIT.SEL = 0U;

  /* brightness setting correction all midpoint */
  GLCDC.BRIGHT1.BIT.BRTG = 512U;
  GLCDC.BRIGHT2.BIT.BRTB = 512U;
  GLCDC.BRIGHT2.BIT.BRTR = 512U;

  /* contrast setting correction all midpoint */
  GLCDC.CONTRAST.BIT.CONTG = 128U;
  GLCDC.CONTRAST.BIT.CONTB = 128U;
  GLCDC.CONTRAST.BIT.CONTR = 128U;

  /* disable gamma correction */
  GLCDC.GAMSW.BIT.GAMON = 0U;

  /* set the GLCD detections and interrupts */

  /* disable interrupt group Al1 */
  IEN(ICU, GROUPAL1) = 0U;

  /* enable VPOS line notification and interrupt */
  GLCDC.DTCTEN.BIT.VPOSDTC = 1U;
  GLCDC.INTEN.BIT.VPOSINTEN = 1U;
  EN(GLCDC, VPOS) = 1U;

  /* program line number to first line in front porch */
  GLCDC.GR2CLUTINT.BIT.LINE = LCD_VERT_SYNC_AREA + LCD_VERT_BACK_PORCH
    + LCD_VERT_ACTIVE_AREA + 1;

  /* disable gr1 underflow notification and interrupt*/
  GLCDC.DTCTEN.BIT.GR1UFDTC = 0U;
  GLCDC.INTEN.BIT.GR1UFINTEN = 0U;
  EN(GLCDC, GR1UF) = 0U;

  /* disable gr2 underflow notification and disable interrupt*/
  GLCDC.DTCTEN.BIT.GR2UFDTC = 0U;
  GLCDC.INTEN.BIT.GR2UFINTEN = 0U;
  EN(GLCDC, GR2UF) = 0U;

  /* set interrupt group Al1 priority */
  IPR(ICU, GROUPAL1) = 15U;

  /* enable interrupt group Al1 */
  IEN(ICU, GROUPAL1) = 1U;

  /* allow reading of frame buffer 2 */
  GLCDC.GR2FLMRD.BIT.RENB = 1U;

  /* release from software reset */
  GLCDC.BGEN.BIT.SWRST = 1U;

  /* enable background generating block */
  GLCDC.BGEN.BIT.EN = 0U;

  /* set current frame buffer */
  GLCDC.GR2AB1.BIT.DISPSEL = 2U;

  /* enable background generating block */
  GLCDC.BGEN.BIT.EN = 1U;

  /* enable register reflection */
  GLCDC.BGEN.BIT.VEN = 1U;

  /* check and clean display info structure */
  if ( !aDisplayInfo )
    return 0;
  memset( aDisplayInfo, 0, sizeof( XDisplayInfo ));

  /* return the current display configuration */
  aDisplayInfo->FrameBuffer    = (void*)EW_FRAME_BUFFER_ADDR;
  aDisplayInfo->DoubleBuffer   = (void*)EW_DOUBLE_BUFFER_ADDR;
  aDisplayInfo->BufferWidth    = EW_FRAME_BUFFER_WIDTH;
  aDisplayInfo->BufferHeight   = EW_FRAME_BUFFER_HEIGHT;
  aDisplayInfo->DisplayWidth   = EW_DISPLAY_WIDTH;
  aDisplayInfo->DisplayHeight  = EW_DISPLAY_HEIGHT;

  #if EW_USE_DOUBLE_BUFFER == 1
    aDisplayInfo->UpdateMode   = EW_BSP_DISPLAY_UPDATE_NORMAL;
  #else
    aDisplayInfo->UpdateMode   = EW_BSP_DISPLAY_UPDATE_PARTIAL;
  #endif

  return 1;
}


/*******************************************************************************
* FUNCTION:
*   EwBspDisplayDone
*
* DESCRIPTION:
*   The function EwBspDisplayDone deinitializes the display hardware.
*
* ARGUMENTS:
*   aDisplayInfo - Display info data structure.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspDisplayDone( XDisplayInfo* aDisplayInfo )
{
  EW_UNUSED_ARG( aDisplayInfo );
}


/*******************************************************************************
* FUNCTION:
*   EwBspDisplayGetUpdateArea
*
* DESCRIPTION:
*   The function EwBspDisplayGetUpdateArea returns the next update area
*   depending on the selected display mode:
*   In case of a synchroneous single-buffer, the function has to return the
*   the rectangular areas that correspond to the horizontal stripes (fields)
*   of the framebuffer.
*   In case of a scratch-pad buffer, the function has to return the subareas
*   that fit into the provided update rectangle.
*   During each display update, this function is called until it returns 0.
*
* ARGUMENTS:
*   aUpdateRect - Rectangular area which should be updated (redrawn).
*
* RETURN VALUE:
*   Returns 1 if a further update area can be provided, 0 otherwise.
*
*******************************************************************************/
int EwBspDisplayGetUpdateArea( XRect* aUpdateRect )
{
  #if EW_USE_DOUBLE_BUFFER == 0

    static int field = 0;

    /* check if last field is already reached */
    if ( field >= EW_NUMBER_OF_FIELDS )
    {
      field = 0;
      return 0;
    }

    /* determine rectangular area of current field */
    #if EW_SURFACE_ROTATION == 0
      *aUpdateRect = EwNewRect( 0, field * EW_FRAME_BUFFER_HEIGHT / EW_NUMBER_OF_FIELDS,
        EW_FRAME_BUFFER_WIDTH, ( field + 1 ) * EW_FRAME_BUFFER_HEIGHT / EW_NUMBER_OF_FIELDS );
    #endif

    #if EW_SURFACE_ROTATION == 90
      *aUpdateRect = EwNewRect( field * EW_FRAME_BUFFER_HEIGHT / EW_NUMBER_OF_FIELDS, 0,
        ( field + 1 ) * EW_FRAME_BUFFER_HEIGHT / EW_NUMBER_OF_FIELDS, EW_FRAME_BUFFER_WIDTH );
    #endif

    #if EW_SURFACE_ROTATION == 180
      *aUpdateRect = EwNewRect( 0, EW_FRAME_BUFFER_HEIGHT - ( field + 1 ) * EW_FRAME_BUFFER_HEIGHT / EW_NUMBER_OF_FIELDS,
        EW_FRAME_BUFFER_WIDTH, EW_FRAME_BUFFER_HEIGHT - field * EW_FRAME_BUFFER_HEIGHT / EW_NUMBER_OF_FIELDS );
    #endif

    #if EW_SURFACE_ROTATION == 270
      *aUpdateRect = EwNewRect( EW_FRAME_BUFFER_HEIGHT - ( field + 1 ) * EW_FRAME_BUFFER_HEIGHT / EW_NUMBER_OF_FIELDS,
        0, EW_FRAME_BUFFER_HEIGHT - field * EW_FRAME_BUFFER_HEIGHT / EW_NUMBER_OF_FIELDS, EW_FRAME_BUFFER_WIDTH );
    #endif

    /* next field */
    field++;

    /* sync on start line of next field to ensure save drawing operation */
    SyncOnLine( field * EW_FRAME_BUFFER_HEIGHT / EW_NUMBER_OF_FIELDS );

    return 1;

  #endif

  return 0;
}


/*******************************************************************************
* FUNCTION:
*   EwBspDisplayWaitForCompletion
*
* DESCRIPTION:
*   The function EwBspDisplayWaitForCompletion is called from the Graphics Engine
*   to ensure that all pending activities of the display system are completed, so
*   that the rendering of the next frame can start.
*   In case of a double-buffering system, the function has to wait until the
*   V-sync has occured and the pending buffer is used by the display controller.
*   In case of an external display controller, the function has to wait until
*   the transfer (update) of the graphics data has been completed and there are
*   no pending buffers.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspDisplayWaitForCompletion( void )
{
  #if EW_USE_DOUBLE_BUFFER == 1

    CPU_LOAD_SET_IDLE();

    while( PendingFramebuffer )
      ;

    CPU_LOAD_SET_ACTIVE();

  #endif
}


/*******************************************************************************
* FUNCTION:
*   EwBspDisplayCommitBuffer
*
* DESCRIPTION:
*   The function EwBspDisplayCommitBuffer is called from the Graphics Engine
*   when the rendering of a certain buffer has been completed.
*   The type of buffer depends on the selected framebuffer concept.
*   If the display is running in a double-buffering mode, the function is called
*   after each buffer update in order to change the currently active framebuffer
*   address. Changing the framebuffer address should be synchronized with V-sync.
*   If the system is using an external graphics controller, this function is
*   responsible to start the transfer of the framebuffer content.
*
* ARGUMENTS:
*   aAddress - Address of the framebuffer to be shown on the display.
*   aX,
*   aY       - Origin of the area which has been updated by the Graphics Engine.
*   aWidth,
*   aHeight  - Size of the area which has been updated by the Graphics Engine.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspDisplayCommitBuffer( void* aAddress, int aX, int aY, int aWidth, int aHeight )
{
  #if EW_USE_DOUBLE_BUFFER == 1

    /* BGEN.VEN bit and GR2VEN.VEN bit has to be 0 */
    while (( GLCDC.BGEN.BIT.VEN == 1 ) || ( GLCDC.GR2VEN.BIT.VEN == 1))
  	;

    /* set start address of GR2 buffer */
    GLCDC.GR2FLM2 = (uint32_t)aAddress;

    /* enable register reflection (bit is cleared automatically) */
    GLCDC.GR2VEN.BIT.VEN = 1;

    /* set pending framebuffer address to be used on next V-sync */
    PendingFramebuffer = (uint32_t)aAddress;

  #else

    /* set start address of gr2 buffer */
    GLCDC.GR2FLM2 = (uint32_t)aAddress;

  #endif
}


/*******************************************************************************
* FUNCTION:
*   EwBspDisplaySetClut
*
* DESCRIPTION:
*   The function EwBspDisplaySetClut is called from the Graphics Engine
*   in order to update the hardware CLUT of the current framebuffer.
*   The function is only called when the color format of the framebuffer is
*   Index8 or LumA44.
*
* ARGUMENTS:
*   aClut - Pointer to a color lookup table with 256 entries.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspDisplaySetClut( unsigned long* aClut )
{
  int i;

  for ( i = 0; i < 256; i++ )
  {
    GLCDC.GR2CLUT0[ i ].LONG = aClut[ i ];
  }

  GLCDC.GR2CLUTINT.BIT.SEL = 0;
}

/* tzi, msy */
